"use strict"
module.exports = {
  en: require("./en/jtd"),
  ca: require("./ca/jtd"),
  cs: require("./cs/jtd"),
  de: require("./de/jtd"),
  es: require("./es/jtd"),
  fi: require("./fi/jtd"),
  fr: require("./fr/jtd"),
  hu: require("./hu/jtd"),
  id: require("./id/jtd"),
  it: require("./it/jtd"),
  ja: require("./ja/jtd"),
  ko: require("./ko/jtd"),
  nb: require("./nb/jtd"),
  nl: require("./nl/jtd"),
  pl: require("./pl/jtd"),
  "pt-BR": require("./pt-BR/jtd"),
  ru: require("./ru/jtd"),
  sk: require("./sk/jtd"),
  sv: require("./sv/jtd"),
  th: require("./th/jtd"),
  zh: require("./zh/jtd"),
  "zh-TW": require("./zh-TW/jtd"),
}
