"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.SchemaDialect = void 0;
var SchemaDialect;
(function (SchemaDialect) {
    SchemaDialect["draft04"] = "draft04";
    SchemaDialect["draft07"] = "draft07";
    SchemaDialect["draft2019"] = "draft2019-09";
    SchemaDialect["draft2020"] = "draft2020-12";
})(SchemaDialect = exports.SchemaDialect || (exports.SchemaDialect = {}));
//# sourceMappingURL=jsonSchema.js.map