import { Connection } from 'vscode-languageserver';
import { CommandExecutor } from '../../languageserver/commandExecutor';
export declare function registerCommands(commandExecutor: CommandExecutor, connection: Connection): void;
