import { Localize } from "./types"
type Language =
  | "en"
  | "ca"
  | "cs"
  | "de"
  | "es"
  | "fi"
  | "fr"
  | "hu"
  | "id"
  | "it"
  | "ja"
  | "ko"
  | "nb"
  | "nl"
  | "pl"
  | "pt-BR"
  | "ru"
  | "sk"
  | "sv"
  | "th"
  | "zh"
  | "zh-TW"
declare const def: Record<Language, Localize>
export default def
