import { Parser, Printer } from "../index.js";

export declare const parsers: {
  angular: Parser;
  html: Parser;
  lwc: Parser;
  mjml: Parser;
  vue: Parser;
};

export declare const printers: {
  html: Printer;
};
